@@ -11,7 +11,7 @@
 #include <features.h>
 #include <sys/types.h>
 #include <sys/socket.h>
-#include <arpa/inet.h>
+#include "platform.h"
 
 
 extern const char bb_INET_default[];    /* = "default" */
@@ -29,7 +29,5 @@ extern int INET_resolve(const char *name, struct sockaddr_in *s_in, int hostfirs
 extern int INET_rresolve(char *name, size_t len, struct sockaddr_in *s_in,
 			 int numeric, unsigned int netmask);
 
-#ifdef CONFIG_FEATURE_IPV6
 extern int INET6_resolve(const char *name, struct sockaddr_in6 *sin6);
 extern int INET6_rresolve(char *name, size_t len, struct sockaddr_in6 *sin6, int numeric);
-#endif
