@@ -2,9 +2,7 @@
 /*
  * Utility routines.
  *
- * Copyright (C) tons of folks.  Tracking down who wrote what
- * isn't something I'm going to worry about...  If you wrote something
- * here, please feel free to acknowledge your work.
+ * Copyright (C) 1999-2004 by Erik Andersen <andersen@codepoet.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,26 +17,18 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
- * Based in part on code from sash, Copyright (c) 1999 by David I. Bell 
- * Permission has been granted to redistribute this code under the GPL.
- *
  */
 
 #include <stdio.h>
 #include "libbb.h"
 
 
-/* Busybox mount uses either /proc/mounts or /dev/mtab to 
- * get the list of currently mounted filesystems */ 
-#if defined BB_FEATURE_MTAB_SUPPORT
-const char mtab_file[] = "/etc/mtab";
+/* Busybox mount uses either /proc/mounts or /etc/mtab to
+ * get the list of currently mounted filesystems */
+#if defined(CONFIG_FEATURE_MTAB_SUPPORT)
+const char bb_path_mtab_file[] = "/etc/mtab";
 #else
-#  if defined BB_FEATURE_USE_DEVPS_PATCH
-      const char mtab_file[] = "/dev/mtab";
-#  else
-      const char mtab_file[] = "/proc/mounts";
-#  endif
+const char bb_path_mtab_file[] = "/proc/mounts";
 #endif
 
 
